#include <stdio.h>
#include <sybfront.h>
#include <sybdb.h>
#include "sybdbex.h"
#include <wiscbench.h>

/*
 * wisc_query27.c
 *
 * Wisconsin Benchmark - Delete 1 Tuple
 *                       Query 27 (no index: use -n flag) or
 *                       Query 30 (indexed: use -i flag )
 *                                
 *	NOTE: if -n used, then for valid results, this query must be run
 *		before running "wisc_index"
 *
 * Implemented by David DeWitt and Michael Franklin
 * 
 * Copyright (c) 1991 David DeWitt and Michael Franklin
 *
 */

main (argc,argv)
    char	**argv;

{
    /* sybase specific variables */
    DBPROCESS       *dbproc;
    RETCODE        result_code; /* to hold the results of dbresults(). */
    STATUS         row_code;    /* to hold the results of dbnextrow(). */

    char	*username;
    char	*passwd;
    char	*optstring;
    extern 	char *optarg;
    extern 	int optind, opterr;
    int		c;
    int		errflg;
    int		no_index;

    int		i;
    int		value;
    int		tupCount; /* number of tuples in the relation */

    char	query[1000];
    int 	starttime, endtime, totaltime;
    int 	numtrials,totalrows,trialrows;

    username = passwd = NULL;
    errflg = 0;
    /* process the command line arguments */
    while ((c = getopt(argc,argv,"u:p:ni")) != EOF)
    {
	switch (c) {
	case 'u':	// get user name
	    username = optarg; 
	    break;
	case 'p':	// get user passwd
	    passwd = optarg;
	    break;
	case 'n':	// no indexing specified
	    no_index = TRUE;
	    break;
	case 'i':	// indexing specified
	    no_index = FALSE;
	    break;
	case '?':
	    errflg++;
	}
	if (errflg) 
	{
	     fprintf(stderr,"Must specifiy -n (no index) or -i (indexed) flag.\n");
	     usage();
	     exit (2);
	}
    }
    if (no_index) {
	     fprintf(stderr,"\nQuery 27 (NoIx, Delete 1 Tuple)...\n");
    }
    else {
	     fprintf(stderr,"\nQuery 30 (Indexed, Delete 1 Tuple)...\n");
    }

    /* next open the database */
    dbproc = open_db(DATABASE, username, passwd,FALSE);

    /* run query 27 or 30 - Delete 1 tuple*/

    /* set base constants */
    tupCount = BASERELSIZE; /* from wiscbench.h */

    numtrials = 0;
    totaltime = 0;
    totalrows = 0;

    for (i=0;i<SELREPEATCNT/2;i++)
    {
	value = tupCount+i+1;  /* generate a new unique value */
	/* run SELREPEATCNT queries two at a time */

	/* first do one relation */
	sprintf(query, "delete from %s where unique2D = %d",
    	BASERELATION1,value);
	if (VERBOSE) printf("next query is: %s\n",query);
	dbcmd(dbproc,query); /* form the query */

        /* Send the command buffer to SQL Server for execution. */
	starttime = time1000();
	dbsqlexec(dbproc);
        while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 

	/* then do the second relation */
	sprintf(query, "delete from %s where unique2E = %d",
    	BASERELATION2,value);

	if (VERBOSE) printf("next query is: %s\n",query);
	dbcmd(dbproc,query); /* form the query */

    	/* Send the command buffer to SQL Server for execution. */
    	starttime = time1000();
    	dbsqlexec(dbproc);
    	while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;;
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 
    }
    if (no_index) {
	    print_result("Query 27 (NoIx, Delete 1 Tuple)",
	    numtrials, totalrows, totaltime);
    }
    else {
	    print_result("Query 30 (ClusterIx, Delete 1 Tuple)",
	    numtrials, totalrows, totaltime);
    }

    closedb(dbproc);
    exit(STDEXIT);
}

/* =================================================================== */

usage ()
{
    fprintf (stderr,"\nUsage: wisc_query27 <-n or -i> -u username -p passwd\n");
    fprintf (stderr,"\t-n: There are no indices built on the database\n");
    fprintf (stderr,"\t-i: The database is indexed\n");
    fprintf (stderr,"\t-p: Show query plan\n");
    exit (1);
}
