#include <stdio.h>
#include <sybfront.h>
#include <sybdb.h>
#include "sybdbex.h"
#include <wiscbench.h>

char *malloc();

/*
 * wisc_query32.c
 *
 * Wisconsin Benchmark - Query 32, Update Key Attribute, Non-clustered index
 *                                
 * Implemented by David DeWitt and Michael Franklin
 * 
 * Copyright (c) 1991 David DeWitt and Michael Franklin
 *
 */

main (argc,argv)
    char	**argv;

{
    /* sybase specific variables */
    DBPROCESS       *dbproc;
    RETCODE        result_code; /* to hold the results of dbresults(). */
    STATUS         row_code;    /* to hold the results of dbnextrow(). */

    char	*username;
    char	*passwd;
    char	*optstring;
    extern 	char *optarg;
    extern 	int optind, opterr;
    int		c;
    int		errflg;

    int		i,j;
    int		*values;
    int		tupCount; /* number of tuples in the relation */

    char	query[1000];
    int 	starttime, endtime, totaltime;
    int 	numtrials,totalrows,trialrows;

    username = passwd = NULL;
    errflg = 0;
    /* process the command line arguments */
    while ((c = getopt(argc,argv,"u:p:")) != EOF)
    {
	switch (c) {
	case 'u':	// get user name
	    username = optarg; 
	    break;
	case 'p':	// get user passwd
	    passwd = optarg;
	    break;
	case '?':
	    errflg++;
	}
	if (errflg) 
	{
	     fprintf(stderr,"Must specifiy -n (no index) or -i (indexed) flag.\n");
	     usage();
	     exit (2);
	}
    }

    fprintf(stderr,"\nQuery 32 (Non-ClusterIx, Update Key Attribute)...\n");
    /* next open the database */
    dbproc = open_db(DATABASE, username, passwd,FALSE);

    /* run query 32 - Update Key Attribute*/

    /* set base constants */
    tupCount = BASERELSIZE; /* from wiscbench.h */
    numtrials = 0;
    totaltime = 0;
    totalrows = 0;
    values = (int*) malloc(SELREPEATCNT * sizeof(int));
    if (values == NULL) printf("malloc failed\n");

    /* first we go through modified the tuples selected */
    /* after running our timing tests we reset the attributes */
    /* to their original values */
    for (i=0, j=-1;i<SELREPEATCNT/2;i++)
    {
	/* run SELREPEATCNT queries two at a time */

	/* first do one relation */
	values[++j] = (int) random()%tupCount;
	sprintf(query, "update %s set unique1D = %d where unique1D = %d",
		BASERELATION1, values[j]+tupCount, values[j]);
	if (VERBOSE) printf("next query is: %s\n",query);
	dbcmd(dbproc,query); /* form the query */

        /* Send the command buffer to SQL Server for execution. */
	starttime = time1000();
	dbsqlexec(dbproc);
        while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 

	values[++j] = (int) random()%tupCount;
	sprintf(query, "update %s set unique1E = %d where unique1E = %d",
		BASERELATION2, values[j]+tupCount, values[j]);
	if (VERBOSE) printf("next query is: %s\n",query);
	dbcmd(dbproc,query); /* form the query */

    	/* Send the command buffer to SQL Server for execution. */
    	starttime = time1000();
    	dbsqlexec(dbproc);
    	while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;;
    	endtime = time1000();

	/* update running statistics */
    	totaltime += (endtime-starttime);
    	trialrows = get_row_count(dbproc);
    	totalrows += trialrows;
    	print_trial(++numtrials,trialrows,(endtime-starttime)); 
    }
    /* now reset all the tuples to their original values */
    for (i=0, j=0;i<SELREPEATCNT/2;i++)
    {
	/* first do one relation */
	sprintf(query, "update %s set unique1D = %d where unique1D = %d",
		BASERELATION1, values[j], values[j]+tupCount);
	dbcmd(dbproc,query); /* form the query */
	dbsqlexec(dbproc);
        while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;
	j++;

	sprintf(query, "update %s set unique1E = %d where unique1E = %d",
		BASERELATION2, values[j], values[j]+tupCount);
	dbcmd(dbproc,query); /* form the query */
    	dbsqlexec(dbproc);
    	while ((result_code = dbresults(dbproc)) != NO_MORE_RESULTS) continue;;
	j++;
    }

    print_result("Query 32 (Non-ClusterIx, Update Key)",
		 numtrials, totalrows, totaltime);
    closedb(dbproc);
    exit(STDEXIT);
}

/* =================================================================== */

usage ()
{
    fprintf (stderr,"\nUsage: wisc_query28 <-n or -i> -u username -p passwd\n");
    fprintf (stderr,"\t-n: There are no indices built on the database\n");
    fprintf (stderr,"\t-i: The database is indexed\n");
    fprintf (stderr,"\t-p: Show query plan\n");
    exit (1);
}
